statemachine class CKNGWISP extends W3UsableItem {
 
    private var wispCurrentRotationCircleAngle 								: float; 
    private var wispCurrentRotation 												: EulerAngles; 
    private var wispCurrentPosition 													: Vector;
    private var wispCurrentVelocity 													: Vector;
    private var wispCurrentAcceleration 											: Vector;
	
	private var wispGoalPosition 														: Vector;
    private var wispGoalPositionOffset 												: Vector;
    private var wispGoalPositionOffset_regenerateInSeconds 				: float;
   
    private var entity                          											:  CEntity;
    private var entityTemplate                  										:  CEntityTemplate;
    private var resourcePath                   											:  string;
    
    private var active 																		: bool;
   
	editable var factAddedOnUse 														: string;
	editable var factValue 																: int;
	editable var factTimeValid 															: int;
	editable var removeFactOnHide 													: bool;
   
    //event called when item is used
    event OnUsed(usedBy : CEntity) {
	
		super.OnUsed(usedBy);
		
		//resource to load
		resourcePath = "dlc\dlckngwisp\data\w2ent\kngwisp.w2ent";
 
		//entityTemplate
        entityTemplate = (CEntityTemplate)LoadResource(resourcePath,true);
		
		//check if used by the Player and that fact does not exist, then proceed with instantiating the entity	
		if(usedBy == thePlayer && !FactsDoesExist("kngwisp_used")) {
			
			//check if entity is already active (instantiated) then, if already instantiated (active), set active to false and proceed with destroying entity
			if(active) {
			
				active = false;				
			}
			
			//entity is instatiated
			active = true;
			
			//add facts (set in kngwisp.w2ent) factAddedOnUse = kngwisp_used, factValue = 1, factTimeValid = -1 
			FactsAdd(factAddedOnUse, factValue, factTimeValid);
			
			//instantiate entity from entityTemplate
			entity = (CKNGWISP)theGame.CreateEntity(entityTemplate, thePlayer.GetWorldPosition() + Vector(0,0,2), thePlayer.GetWorldRotation());
			
			//fail-safe, add a tag to the entity this should also be received from the .w2ent, but CDPR
			entity.AddTag('kngwisp');
			
			// play cookedEffect wisp_fx (particleEffect), hooked also from .w2ent
			entity.PlayEffectSingle('wisp_fx');
			
			//if active is set to false proceed with destroying entity
		} else if (!active) {
				
			FactsRemove(factAddedOnUse);
			entity = theGame.GetEntityByTag('kngwisp');
			entity.RemoveTimer('FollowPlayer');
			entity.StopAllEffects();
			entity.DestroyAfter(0.1);
		}
    }
	
	//event called when entity is destroyed // apperently, also when item event OnHidden is called, not needed tho
	event OnDestroyed() {	
			
	}
	
	//event called when entity is instantiated
	event OnSpawned(spawnData : SEntitySpawnData) {
		
		//declare Vector for l_hand bonePosition
		var bonePos : Vector;
		
		//get position of left hand (boneindex for l_hand is 24)
		bonePos = MatrixGetTranslation(thePlayer.GetBoneWorldMatrixByIndex(24));
		
		//start timer function FollowPlayer (magic.gif)
		this.AddTimer('FollowPlayer', 0.0001, true);
		
		//set initial wisp Position to left hand, with slight offset
		wispCurrentPosition = bonePos + Vector(0,0,0.1);
		//set initial wisp vecolcity, so that it spawns in an "upward" motion away from left_hand
        wispCurrentVelocity = Vector(3,0,0);
		wispCurrentAcceleration = Vector(0,0,0);
		
	}
   
   //DEBUG
    timer function CheckIfFactExists(deltaTime : float, id : int) {
   
        if(FactsDoesExist("kngwisp_used")) {
       
            theGame.GetGuiManager().ShowNotification("Fact does exist!");
			
        } else {
			
			theGame.GetGuiManager().ShowNotification("Fact does not exist!");
		
		}
    }
		   
    timer function FollowPlayer(deltaTime : float, id : int) {
		
		var target : CActor;
		var targetPosition : Vector;
		var targetRotation : EulerAngles;
		
                       
        var playerPosition : Vector;
        var playerRotation : EulerAngles;      
        
        var goalAcceleration : Vector;
        var navigationComputeZReturn : float;  
 
        // DEFAULT CONFIG start //
 
        var rotationCircleRadius : float;
        var rotationCircleMovementSpeed : float;
        var selfRotationSpeed : float;
        var maxAcceleration : float;      
        var accelerationMultiplier : float;
        var maxVelocity : float;
        var velocityDampeningFactor : float;
 
        rotationCircleMovementSpeed = 25;
        rotationCircleRadius = 0.75;
        selfRotationSpeed = 0.3;
        maxAcceleration = 100;
        accelerationMultiplier = 20;
        maxVelocity = 20;
        velocityDampeningFactor = 0.9;
 
        // DEFAULT CONFIG end //
		
		
		// Player is in Interior (buildings, caves etc.)
		// INTERIOR CONFIG start //
		if(thePlayer.IsInInterior()) {
		
		rotationCircleMovementSpeed = 25;
		rotationCircleRadius = 0.45;
        selfRotationSpeed = 0.3;
        maxAcceleration = 100;
        accelerationMultiplier = 20;
        maxVelocity = 20;
        velocityDampeningFactor = 0.9;
		
		}
		// INTERIOR CONFIG end //
       
        // increase current angle in the circle around player
        wispCurrentRotationCircleAngle += deltaTime * rotationCircleMovementSpeed;
 
        // if rotation increased over 360 degrees make sure its under 360
        while(wispCurrentRotationCircleAngle > 360) {          
            wispCurrentRotationCircleAngle -= 360;                        
        }
       
	   // get current target
	   target = thePlayer.GetTarget();
	   // current target position
	   targetPosition = target.GetWorldPosition();
	   // current target rotation
	   targetRotation = target.GetWorldRotation();
	   
	   //get current player position
       playerPosition = thePlayer.GetWorldPosition();
	   //get current player rotation
       playerRotation = thePlayer.GetWorldRotation();
       
        // calculate wisp Goal position

		// if player has target and is in focusmode (right mouse button), change some of the wisps config values to adjust movement around target	
		if(target && theGame.IsFocusModeActive()) {			
			
			rotationCircleMovementSpeed = 500;
			rotationCircleRadius = 0.65;
			maxAcceleration = 200;
			
			wispGoalPosition = targetPosition;
			
				
		} else {
			
			//if no target and no focusmode, stick to player and use default config values	
			wispGoalPosition = playerPosition;
		
		}
		
	    wispGoalPosition += wispGoalPositionOffset;
        wispGoalPosition.X += CosF(Deg2Rad(wispCurrentRotationCircleAngle)) * rotationCircleRadius;
        wispGoalPosition.Y += SinF(Deg2Rad(wispCurrentRotationCircleAngle)) * rotationCircleRadius;
        wispGoalPosition.Z += 2;
       
        // see if we can set new random wisp position offset
        wispGoalPositionOffset_regenerateInSeconds -= deltaTime;
        if(wispGoalPositionOffset_regenerateInSeconds < 0) {
            wispGoalPositionOffset_regenerateInSeconds = RandRangeF(1, 0.1);
            wispGoalPositionOffset = VecRand() * rotationCircleRadius * 0.25;
        }
 
        // accelerate towards goal position
        wispCurrentAcceleration = (wispGoalPosition - wispCurrentPosition) * accelerationMultiplier;
 
        // clamp wisp acceleration, so its not too fast
        if(VecLength(wispCurrentAcceleration) > maxAcceleration) {
            wispCurrentAcceleration = VecNormalize(wispCurrentAcceleration) * maxAcceleration;
        }
 
        // simulate acceleration and velocity for more natural movement
        wispCurrentVelocity += wispCurrentAcceleration * deltaTime;
 
        // dampen velocity
        wispCurrentVelocity *= velocityDampeningFactor;
 
        // clamp wisp velocity, so its not too fast
        if(VecLength(wispCurrentVelocity) > maxVelocity) {
            wispCurrentVelocity = VecNormalize(wispCurrentVelocity) * maxVelocity;
        }
 
        // get the height
        if (theGame.GetWorld().NavigationComputeZ( wispCurrentPosition, wispCurrentPosition.Z - 1, wispCurrentPosition.Z + 1, navigationComputeZReturn ) )
        {
            // if wisp is too close to collision, bounce off it and loose some velocity
            if(AbsF(wispCurrentPosition.Z - navigationComputeZReturn) < 0.1) {
                wispCurrentVelocity.Z *= -0.9;
            }
        }
 
        // simulate acceleration and velocity for more natural movement
        wispCurrentPosition += wispCurrentVelocity * deltaTime;
 
        // if wisp is too far away, just teleport it back to player
        if(VecDistance(wispCurrentPosition, wispGoalPosition) > 100) {
            wispCurrentPosition = wispGoalPosition;
            wispCurrentVelocity = Vector(0,0,0);
            wispCurrentAcceleration = Vector(0,0,0);
        }
 
        // rotate wisp it self over time
        wispCurrentRotation.Pitch += deltaTime * selfRotationSpeed;
 
        // finall set the wisp position and rotation				
		this.TeleportWithRotation(wispCurrentPosition, wispCurrentRotation);
       
    }   
}